<?php
/*
 * Plugin Name: SVG Logo and Text Effects Raster Fallback extension
 * Description: Extends the WordPress SLATE plugin to add raster fallback for browsers that cannot render SVG.
 * Version: 1.0.0
 * Plugin URI: http://www.dashed-slug.net/svg-logo-and-text-effects-extensions/raster-fallbacks-extension
 * Author: Dashed Slug <info@dashed-slug.net>
 * Author URI: http://dashed-slug.net
 * Text Domain: slate-raster
 * Domain Path: /languages/
 * License: GPLv2 or later
 *
 * @package slate-raster
 * @since 1.0.0
 */

// don't load directly
defined( 'ABSPATH' ) || die( '-1' );

include __DIR__ . '/ds-update.php';

if ( ! class_exists( 'SVG_Logo_and_Text_Effects_raster_fallback' ) ) {

	final class SVG_Logo_and_Text_Effects_raster_fallback {

		const OPT_RASTER_FALLBACK = 'slate-opt-raster-fallback';

		private static $_instance = null;

		private static $_errors = array();

		private function __clone() {
			// Cloning disabled
		}

		/**
		 * Creates or returns an instance of this class.
		 *
		 * @return SVG_Logo_and_Text_Effects_raster_fallback
		 */
		public static function get_instance() {
			if ( ! ( self::$_instance instanceof self ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		public static function error_handler( $errno, $errstr, $errfile, $errline, $errcontext ) {
			if ( ! ( error_reporting() & $errno ) ) {
				return;
			}

			// append error to show on admin_notices
			self::$_errors[] = array(
				'errno' => $errno,
				'errstr' => $errstr,
				'errfile' => $errfile,
				'errline' => $errline,
				'errcontext' => $errcontext,
				'current_filter' => current_filter() );

			error_log( __CLASS__ . ": errno: $errno, errstr: $errstr, errfile: $errfile, errline: $errline" );
		}

		private function __construct() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			// Plugin lifecycle hooks
			if ( function_exists( 'register_activation_hook' ) ) {
				register_activation_hook( __FILE__, array( __CLASS__, 'action_activation' ) );
			}

			// Show admin notices
			add_action( 'admin_notices', array( &$this, 'action_admin_notices' ) );

			// show settings link in plugins page
			add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array( &$this, 'action_plugin_action_links' ) );

			// Callbacks for raster fallback
			add_action( 'wp_enqueue_scripts', array( &$this, 'action_wp_enqueue_scripts' ) );
			add_filter( 'body_class', array( &$this, 'filter_body_class' ) );
			add_filter( 'query_vars', array( &$this, 'filter_query_vars' ), 0 );
			add_action( 'init', array( &$this, 'action_init') );
			add_action( 'admin_init', array( &$this, 'action_admin_init') );
			add_action( 'parse_request', array( &$this, 'action_parse_request' ), 0 );
			add_filter( 'slate_output', array( &$this, 'filter_rasterize' ), 10, 2 );

			restore_error_handler();
		}

		public function filter_rasterize( $output, $vars ) {
			if ( get_option( self::OPT_RASTER_FALLBACK ) && class_exists( 'Imagick' ) ) {

				$svg = $vars['svg'];
				$hash = $vars['hash'];

				if ( false === get_transient( "slate-$hash-png" ) ) {

					try {
						$png = new Imagick();
						$png->setBackgroundColor( new ImagickPixel( 'transparent' ) );
						$png->readImageBlob(
							'<?xml version="1.0" encoding="utf-8"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">' . $svg );
						$png->setimageformat( 'png32' );
						$png->setimagecompressionquality( 100 );

						set_transient( "slate-$hash-png", base64_encode( $png->getimageblob() ), HOUR_IN_SECONDS );

					} catch ( Exception $e ) {
						trigger_error(
							__FUNCTION__ . __(
								' Cannot render raster fallback - check that Imagick is installed and that it has its SVG and PNG delegates enabled. ',
								'slate' ) . $e->getMessage(),
							E_USER_NOTICE );
					}
				}

				return $output . '<img src="' . site_url( "index.php?__slate_hash=$hash" ) . "\" class=\"slate raster $hash\" />";

			}
			return $output;
		}

		public function filter_query_vars( $vars ) {
			$vars[] = '__slate_hash';
			return $vars;
		}

		public function action_parse_request() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			global $wp;

			if ( isset( $wp->query_vars['__slate_hash'] ) ) {
				$hash = $wp->query_vars['__slate_hash'];

				if ( ! get_option( self::OPT_RASTER_FALLBACK ) ) {

					error_log( __CLASS__ . ": PNG fallback requested but not enabled. Hash: $hash" );
					header( 'HTTP/1.1 404 Not Found' );
				} else {

					$png = get_transient( "slate-$hash-png" );

					if ( false !== $png ) {
						header( 'Content-type: image/png' );
						echo base64_decode( $png );
					} else {
						error_log( __CLASS__ . ": PNG fallback requested but not found in cache. Hash: $hash" );
						header( 'HTTP/1.1 404 Not Found' );
					}
				}
				exit();
			}
			restore_error_handler();
		}

		public function action_admin_notices() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			if ( ! is_plugin_active( 'slate/slate.php' ) ) {
				echo '<div class="notice notice-warn is-dismissible"><p>' . sprintf(
					__( '<strong>%s</strong> requires the <strong><a href="http://dashed-slug.net/slate" target="_blank">SVG Logo and Text Effects</a></strong> plugin. Please install and activate it on your site.', 'slate-vc' ),
					'SVG Logo and Text Effects Raster Fallback extension' ) . '</p></div>';
			}

			foreach ( self::$_errors as $error ) {
				$level = 'notice';
				if ( E_USER_ERROR == $error['errno'] ) {
					$level = 'error';
				} elseif ( E_USER_WARNING == $error['errno'] ) {
					$level = 'warning';
				}

				echo "<div class=\"notice notice-$level is-dismissible\">";
				echo "<p style=\"font-size: larger;\">{$error['errstr']}</p><pre>" . 'SVG Logo and Text Effects Raster Fallback extension' .
					" on action <code>{$error['current_filter']}</code>, <code>{$error['errfile']}</code>, line <code>{$error['errline']}</code>.</pre></div>";
			}
			restore_error_handler();
		}

		public function action_plugin_action_links( $links ) {
			$links[] = '<a href="' . admin_url( 'options-general.php?page=slate_settings_page' ) . '">'
				. __( 'Settings', 'slate' ) . '</a>';
				return $links;
		}

		public function filter_body_class( $classes ) {
			// Only add classes on frontend.
			if ( is_admin() ) {
				return $classes;
			}

			// Add class to indicate raster fallbacks are enabled.
			if ( get_option( self::OPT_RASTER_FALLBACK ) && class_exists( 'Imagick' ) ) {
				$classes[] = 'slate-raster-fallback';
			}
			return $classes;
		}

		public static function action_activation() {
			add_option( self::OPT_RASTER_FALLBACK, 'on' );
		}

		public function action_init() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			// API endpoint for retrieving raster fallback
			add_rewrite_rule( 'slate/cache/([a-fA-F0-9]+)\.png', 'index.php?__slate_hash=$matches[1]', 'top' );

			restore_error_handler();
		}

		public function action_admin_init() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			register_setting( 'slate_settings_page', self::OPT_RASTER_FALLBACK );

			add_settings_field(
				self::OPT_RASTER_FALLBACK,
				'raster fallback (PNG)',
				array( &$this, 'cb_settings_raster_fallback' ),
				'slate_settings_page',
				'slate_settings_section' );

				restore_error_handler();
		}

		public function cb_settings_raster_fallback() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			$imagick = false;
			if ( class_exists( 'Imagick' ) ) {
				try {
					$image = new Imagick();
					$imagick = $image instanceof Imagick;
				} catch ( Exception $e ) {
					trigger_error( __( 'Imagick not available. Cannot render raster fallback.', 'slate' ), E_USER_NOTICE );
				}
			}
			echo '<input name="' . self::OPT_RASTER_FALLBACK . '" id="' . self::OPT_RASTER_FALLBACK .
			'" type="checkbox" ';

			if ( ! $imagick ) {
				echo 'disabled="disabled" ';
			}

			checked( 'on', get_option( self::OPT_RASTER_FALLBACK ) );

			echo '/>';

			if ( $imagick ) {
				echo __(
					'Enable if you wish to generate PNG fallback images for browsers that don\'t support inline SVG. When disabled, a plain HTML fallback is shown.',
					'slate' );
			} else {
				echo __( 'Imagick not available. Cannot render raster fallback.', 'slate' );
			}

			restore_error_handler();
		}

		public function action_wp_enqueue_scripts() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			if ( ! ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {

				if ( ! is_admin() ) {

					if ( file_exists( __DIR__ . '/assets/scripts/slate-raster.min.js' ) ) {
						$slate_raster_script = 'assets/scripts/slate-raster.min.js';
					} else {
						$slate_raster_script = 'assets/scripts/slate-raster.js';
					}

					wp_enqueue_script(
						'slate-raster',
						plugins_url( $slate_raster_script, __FILE__ ),
						array( 'webfont', 'jquery', 'slate' ),
						false,
						true );

					if ( file_exists( __DIR__ . '/assets/styles/slate-raster.min.css' ) ) {
						$slate_raster_style = 'assets/styles/slate-raster.min.css';
					} else {
						$slate_raster_style = 'assets/styles/slate-raster.css';
					}

					wp_enqueue_style( 'slate-raster', plugins_url( $slate_raster_style, __FILE__ ) );


				}
			}

			restore_error_handler();
		}

	}
}


SVG_Logo_and_Text_Effects_raster_fallback::get_instance();
